#
#            RiSearch
#
# web search engine, version 1.0.01
# (c) Sergej Tarasov, 2000-2004
#
# Homepage: http://risearch.org/
# email: risearch@risearch.org

package lib::spider_lib;
use lib::common_lib;
use lib::index_lib;
use LWP::UserAgent;
use URI::URL;
use HTML::LinkExtor;
use strict;
use riconfig;
use vars qw(@ISA @EXPORT);
no warnings;


use Exporter;
@ISA = ('Exporter');
@EXPORT = qw(
             &extract_links &start_spider &index_page
             
             );

use vars  qw(
             );




#=====================================================================
#
#    Function: start_spider
#    Last modified: 24.03.2005 18:34
#
#=====================================================================

sub start_spider {
    
    my ($to_visit, $func_ref, $rules, $login, $password, $action) = @_;
    my %to_visit = %{$to_visit};
    
    $rules = create_rule($rules);

    my $get_url; my $hdrs; my $url; my $req; my $ua; my $resp;
    my $data; my $lastmodified; my $BASE;
    my %visited_url;

    my $index; my $follow;
    
    # Start User-Agent
    my $ua = start_ua();
    

while (1) {
    
    my $cur_depth = 0;
    
    ($get_url,$cur_depth) = each %to_visit;
    
    
    if ( ! defined $get_url ) { last }
    if ($get_url eq "") { next }
    if (exists($visited_url{$get_url})) { next }
    
    ($index,$follow) = &$rules($get_url);
        
    if ( ! $index && ! $follow ) {
        print "Not allowed: $get_url\n" if $cfg{debug_level} >0;
        next;
    }
    
    print "Get: $get_url\n";
    
    my $resp = get_page($get_url,30,$login,$password,$ua);
    my $BASE = $$resp{'base'};
    $visited_url{$get_url}++;



    if ($BASE eq "") {
        delete $to_visit{$get_url};
        print "Error message: ",$$resp{'status_line'},"\n" if $$resp{'status_line'};
        print "\n";
        next;
    }
    if ( ($$resp{'uri'} ne $get_url)  &&  exists($visited_url{$$resp{'uri'}})) { 
        print "URL: $$resp{'uri'}\n" if $cfg{'verbose_output'};
        print "Already visited\n\n";
        delete $to_visit{$get_url};
        next;
    }
    if ($$resp{'uri'} ne $get_url) {
        print "Redirected\n";
        print "URL: $$resp{'uri'}\n" if $cfg{'verbose_output'};
        $visited_url{$$resp{'uri'}}++;
        $get_url = $$resp{'uri'};
    }


    

    if ($$resp{'text'}) {
        my $data = $$resp{'text'};
        my $lastmodified = $$resp{'last_modified'};
        if ( $follow && $cur_depth < $cfg{max_depth} ) {
            foreach my $new_link (extract_links( $data, $BASE, $rules )) {
                if (! exists($visited_url{$new_link})) {

                    my ($tindex,$tfollow) = &$rules($new_link);
                    if ( ! $tindex && ! $tfollow ) { next }
                    if (exists $to_visit{$new_link}) {
                        $to_visit{$new_link} = $cur_depth+1 if $to_visit{$new_link} > $cur_depth+1;
                    } else {
                        $to_visit{$new_link} = $cur_depth+1;
                    }
                    
                }
            }
        }
        
        if ($index) {

            &$func_ref( { text => $data, url => $get_url, date => $lastmodified,
                          content_encoding => $$resp{content_encoding},
                          content_type => $$resp{content_type} } );
        }

    } else {
        print "Error message: ",$$resp{'message'},"\n";
    }
    delete $to_visit{$get_url};
    
    scalar keys %to_visit;
    
    
    print "Done\n\n" if $cfg{'verbose_output'};

    if ($cfg{max_files} && $cfg{max_files} < $cfg{cfn}+1) {
        last;
    }
    
    
    sleep($cfg{'spider_delay'});
}

} 
#=====================================================================
#
#    Function: index_page
#    Last modified: 12.01.2004 17:42
#
#=====================================================================

sub index_page {
 
    my ($get_url, $func_ref, $login, $password) = @_;
    
    my $resp = get_page($get_url,30,$login,$password);
    my $BASE = $$resp{'base'};

    if ($$resp{'content'}) {
        my $data = $$resp{'content'};
        my $lastmodified = $$resp{'last_modified'};
                
        &$func_ref( { text => $data, url => $get_url, date => $lastmodified,
                      content_encoding => $$resp{content_encoding} } );
        
    } else {
        print $$resp{'message'},"\n"; 
    }
   
}
#=====================================================================
#
#    Function: extract_links
#    Last modified: 28.08.2003 14:36
#
#=====================================================================

sub extract_links {
    my $data = shift;
    my $BASE = shift;
    my $rules = shift;
    
    my $linkarray; my $allow_url; 
    my @urls;
   
    
    my $parser = HTML::LinkExtor->new(undef,$BASE);
    $parser->parse($data)->eof;
    my @links = $parser->links;

    foreach $linkarray (@links) {
        my @element = @$linkarray;
        my $el_type = shift @element;
        while (@element) {
            my ($attr_name, $attr_value) = splice(@element, 0, 2);
  
            $attr_value =~ s/#.*?$//;
            if ($cfg{cut_default_filenames_reg} eq 'YES') { $attr_value =~ s|$cfg{default_filenames_reg}|/|io }
            if ($attr_value =~ m|^mailto:|) { next }
            
            if ($el_type eq "a" && $attr_name eq "href") {
            	push(@urls,$attr_value);
            	next;
            }
            if ($el_type eq "frame" && $attr_name eq "src") {
            	push(@urls,$attr_value);
            	next;
            }
            if ($el_type eq "iframe" && $attr_name eq "src") {
            	push(@urls,$attr_value);
            	next;
            }
            if ($el_type eq "area" && $attr_name eq "href") {
            	push(@urls,$attr_value);
            }
        }
    }
    
    return @urls;
    
}
#=====================================================================


1;