#
#            RiSearch
#
# web search engine, version 1.0.01
# (c) Sergej Tarasov, 2000-2004
#
# Homepage: http://risearch.org/
# email: risearch@risearch.org

package riconfig;
use strict;
use Exporter;
no warnings;
use vars qw(@ISA @EXPORT);
@ISA         = qw(Exporter);
@EXPORT      = qw(%cfg @zone @zone_name %stop_words %index_words
                  @attr_def @attr_conf @attr_weight);
use vars qw(%cfg @zone @zone_name %stop_words %index_words @attr_def @attr_conf @attr_weight);
%cfg = ();
%stop_words = ();
%index_words = ();
%cfg = (


#===================================================================#
#===================================================================#
#                                                                   #
#                                                                   #
#                    Set variables below                            #
#                                                                   #
#                                                                   #
#===================================================================#
#===================================================================#

# Directory where yours html files are located
# In most cases you may use path relative to the location of script
# Or use absolute path
# Type "./" for the current directory
base_dir => "..",

# Base URL of your site
base_url => "http://www.search.ru/",


# Starting URL (used by spider)
start_url => [qw(

http://www.search.ru/
http://risearch.com/

)],


# Filter rules
rules => q(

Disallow *
Allow http://www.search.ru/*
Allow http://risearch.com/*
Disallow NoCase */img/* 
Disallow NoMatch *.htm *.html *.txt *.php  */ 

),


# Page fetching delay
spider_delay => 0,

# Follow 'robots.txt' rules
use_robots_txt_rules => 0,

# Webmaster's e-mail
e_mail => 'foo@bar.com',

# Maximal spidering depth
max_depth => 20,

# Maximal number of files to index (0 - no limit)
max_files => 0,

# HTTP authorization
login => "",
password => "",

# HTTP proxy ( in the form "http://server.com:port/" or "http://user:password@server.com:port/" )
proxy => "",


#===================================================================
#
#     All other variables are optional. Script should work fine
#  with default settings.
#     These variables controls the indexing process.
#
#===================================================================

# During indexing process script will print info for every 100th file ("0")
# or for every file ("1")
verbose_output => 1,



# Minimum word length to index
min_length => 3,

# Maximum word length to index. Longer words will be truncated
max_length => 32,

# Maximum document size (bigger files will be truncated)
max_doc_size => 1000000,

# Index or not numbers (set   $numbers = ""   if you don't want to index numbers)
# You may add here other non-letter characters, which you want to index
numbers => '0-9',


# Cut default filenames from URL ("YES" or "NO")
cut_default_filenames => 'YES',
default_filenames => 'index.htm index.html default.htm',

# Convert URL to lower case ("YES" or "NO")
url_to_lower_case => 'NO',

# Index META tags ("YES" or "NO")
use_META => "NO",

# Index IMG ALT tag ("YES" or "NO")
use_ALT => "NO",

# Index URL ("YES" or "NO")
index_url => "YES",

# Delete hyphen at the end of strings ("YES" or "NO")
del_hyphen => "NO",

# List of stopwords ("YES" or "NO")
use_stop_words => "YES",
stop_words => [ qw(
and any are but can had has have her here him his
how its not our out per she some than that the their them then there
these they was were what you
) ],


# List of short words, which should be indexed
index_words => [ qw(
xp pc 3d
) ],



# Write queries to log file ("YES" or "NO")
# Please note, you should create directory "log" by hands
# script will not check, if there exist such directory
# Please edit some parameters in file "stat.pl"
create_log => "YES",


# Date format: "md" - month first, "dm" - day first
date_format => "dm",


#===================================================================
#
#     File parsing configuration.
#
#===================================================================


# list of extensions, were script should not remove HTML tags
non_parse_ext => 'txt',


# Binary files extensions (these files will not be indexed, but
# URL will be indexed)
bin_ext => 'ppt xls',


# Parts of documents, which should not be indexed
# Set "YES" and edit, if you want to use this feature
use_selective_indexing => "NO",
no_index_strings => {
    q[<!-- No index start 1 -->] => q[<!-- No index end 1 -->],
    q[<!-- No index start 2 -->] => q[<!-- No index end 2 -->],
},


#===================================================================
#
#     Relevance control variables
#
#===================================================================


# These variables controls the possible variants of results sorting
# 0 - "NO", 1 - "YES"
allow_sort_by_rating => 1,
allow_sort_by_date   => 1,
allow_sort_by_size   => 1,

# Set the weight for rating calculation
weight_title    => 5,
weight_heading  => 3,
weight_bold     => 2,
weight_italic   => 1,
weight_link     => 3,
weight_keywords => 1,
weight_descr    => 1,

# use word frequency
word_freq => 0,



#===================================================================
#
#     These variables controls the script output.
#
#===================================================================

# Default search type
# 0 - search by beginning (only with INDEXING_SCHEME => 2)
# 1 - exact word search
def_search_type => 1,

# Default search mode
# OR (any word)
# AND (all words)
def_search_mode => "AND",

# Number of results per page
res_num => 10,

# Maximal number of results (0 - no limit)
max_res_found => 0,

# Truncate TITLE of page to 64 characters ("YES" or "NO")
truncate_title => "NO",

# Define length of page description in output
# and use META description ("YES") or first "n" characters of page ("NO")
stored_descr_size => 512,
descr_size => 256,
use_META_descr => "NO",

# Delete these characters from description
del_descr_chars => "",

# URL length limit in output (0 - no limit)
url_length_limit => 0,

# Script will try to read file from disk for highlighting
read_file_from_local_disk => "NO",

# Read only N bytes (0 - no limit)
read_file_from_local_disk_limit => 50000,

# look of highlighted terms is defined by these parameters 
mark_start => "<b style='color:black;background-color:#ffff66'>",
mark_end   => "</b>",

# parsee SSI directives in results template
parse_SSI => 1,

# template files
search_template => "templates/template.htm",
admin_template => "templates/admin.htm",
config_template => "templates/config.htm",
stat_template => "templates/stat_template.htm",


#===================================================================
#
#   Change below only if you need multilanguage support
#   With default settings script will work with
#   English, Russian (win1251 encoding) and most European languages
#
#===================================================================

# Capital letters
CAP_LETTERS => '\xC0-\xDF\xA8',

# Lower case letters
LOW_LETTERS => '\xE0-\xFF\xB8',


# Translate escape chars (like &Egrave; or &#255;) ("YES" or "NO")
use_esc => "YES",

# If you use Unicode characters in your site in the form
# &#NNNN; (where NNNN>255), uncomment and edit below
# Samples for different languages can be found in file unicode.txt

code2char => {
# 1040 => "",
# 1041 => "",
# 1042 => "",
# 1043 => "",
},


#===================================================================
#
#    Automatic recoding configuration
#
#===================================================================

recode_by_http_header => {
#    'windows-1251' => \&lib::common_lib::win2koi,
#    'koi8-r' => \&lib::common_lib::koi2win,
},

recode_by_meta_tag => {
#    'windows-1251' => \&lib::common_lib::win2koi,
#    'koi8-r' => \&lib::common_lib::koi2win,
#    'utf-8' => \&lib::common_lib::utf8_to_win1251,
},

recode_by_url => {
#    '/dir1/' => \&lib::common_lib::win2koi,
#    '/dir2/' => \&lib::common_lib::koi2win,
},


#===================================================================
#
#   Parameters for index configuration
#   
#===================================================================

# database files
# Path to database files
HASH      => "0_hash",
FINFO     => "0_finfo",
FINFO_IND => "0_finfo_ind",
WORD_IND  => "0_word_ind",
RATING    => "0_rating",
FDATE     => "0_fdate",
FSIZE     => "0_fsize",


PASS          => "0_pass",

# directory for database files
db     => "db",

# new database will be created in "db_new" directory and then 
# old database will be replaced by new
create_new_database => "YES",

# old database will not be deleted after new database is created
delete_old_database => "NO",

# Hash table size
# For big site use larger value
# 1 - Tiny    ~1Mb
# 2 - Medium  ~10Mb
# 3 - Big     ~50Mb
# 4 - Large   >100Mb
site_size => 4,

# Indexing scheme
# Whole word - 1
# Beginning of the word - 2
INDEXING_SCHEME => 2,


# 0 - slow indexing, less memory used
# 1 - fast indexing, more memory used
indexing_speed => 1,

# index format (compact index can be used up to 65536 documents)
compact_index => 1,

# memory block size for temporary index
temp_db_size => 10000000,




);


#===================================================================#
#===================================================================#
#                                                                   #
#                                                                   #
#                --- end of configuration ---                       #
#                                                                   #
#  Please do not edit below this line unless you know what you do   #
#                                                                   #
#                                                                   #
#===================================================================#
#===================================================================#





1;